/*
   MicroDexed

   MicroDexed is a port of the Dexed sound engine
   Dexed ist heavily based on https://github.com/google/music-synthesizer-for-an
  droid

   (c)2018-2021 H. Wirtz <wirtz@parasitstudio.de>

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   THE SOFTWARE.
*/

#include <Arduino.h>
#include "config.h"
#include "drumset.h"

#ifndef _DRUMS_H
#define _DRUMS_H

typedef struct drum_config_s {
  uint8_t drum_class; // Type of drum
  uint8_t midinote;   // Triggered by note
  char name[DRUM_NAME_LEN];
  const uint8_t* drum_data;
  char shortname[2];  // 1 char name for sequencer
  uint32_t len;       // number of elements in drum_data
  float32_t pitch;    // variable pitch per note for sequencer 
  float32_t p_offset; // "static" pitch offset to correct root note to root of other samples
  float32_t pan;      // Panorama (-1.0 - +1.0)
  float32_t vol_max;  // max. Volume (0.0 - 1.0)
  float32_t vol_min;  // min. Volume (0.0 - 1.0, should be <= vol_max)
  float32_t reverb_send; // how much signal to send to the reverb (0.0 - 1.0)
} drum_config_t;

enum {DRUM_NONE, DRUM_BASS, DRUM_SNARE, DRUM_HIHAT, DRUM_HANDCLAP, DRUM_RIDE, DRUM_CRASH, DRUM_LOWTOM, DRUM_MIDTOM, DRUM_HIGHTOM, DRUM_PERCUSSION};

// DEFAULT MIDI CHANNEL FOR SAMPLEDRUMS
uint8_t drum_midi_channel = 10;

drum_config_t drum_config[NUM_DRUMSET_CONFIG] =
{
   {
    DRUM_HIHAT,
    210,
    "SQBass",
    SQBass,
    "B",
    32000,
    1.0,
    1.39,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_MIDTOM,
    211,
    "CALLIOP",
    CALLIOP,
    "C",
    45056,
    1.0,
    1.0,
    0.0,
    0.8,
    0.0,
    0.0
  },
  {
    DRUM_CRASH,
    212,
    "CALLIOP",
    CALLIOP,
    "C",
    45056,
    1.0,
    1.0,
    0.0,
    0.8,
    0.0,
    0.0
  },
  {
    DRUM_RIDE,
   213,
    "ARR1",
    ARR1,
    "A",
    21248,
    1.0,
    0.65,
    0.0,
    0.7,
    0.0,
    0.0
  },
  {
    DRUM_BASS,
    214,
    "FVoice",
    FVoice,
    "F",
    39040,
    1.0,
    1.0,
    0.0,
    0.8,
    0.0,
    0.0
  },
  
   {
    DRUM_BASS,
    MIDI_A2,
    "Clap808",
    Clap808,
    "C",
    7744,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_BASS,
    MIDI_B2,
    "S_Kick1",
    S_kick_1,
    "B",
    22208,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_BASS,
    MIDI_C3,
    "S_Kick2",
    S_kick2_8,
    "B",
    22528,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_CIS3,
    "S_Stick",
    S_Stick_1,
    "B",
    13632,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_SNARE,
    MIDI_D3,
    "S_snare1",
    S_snare_Ghost_1,
    "B",
    22272,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_SNARE,
    MIDI_DIS3,
    "S_sna2G",
    S_snare2_Ghost_1,
    "B",
    28032,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_SNARE,
    MIDI_E3,
    "S_snar2",
    S_snare_2,
    "B",
    17920,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_LOWTOM,
    MIDI_F3,
    "S_loTom1",
    S_loTom_1,
    "B",
    58752,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_HIHAT,
    MIDI_FIS3,
    "S_hhCl1",
    S_hhClosed_1,
    "B",
    9280,
    0.0,
    1.0,
    0.0,
    0.7,
    0.0,
    0.0
  },
  {
    DRUM_MIDTOM,
    MIDI_G3,
    "S_lTom1",
    S_loTom_1,
    "B",
    58752,
    1.2,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  { //10
    DRUM_HIHAT,
    MIDI_GIS3,
    "S_hh_o1",
    S_hh_o_1_2,
    "B",
    18240,
    0.0,
    1.0,
    0.0,
    0.7,
    0.0,
    0.0
  },
  {
    DRUM_MIDTOM,
    MIDI_A3,
    "S_hiTom5",
    S_hiTom_5,
    "B",
    38592,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_HIHAT,
    MIDI_AIS3,
    "S_hh_o5",
    S_hh_o_5_4,
    "B",
    98368,
    0.0,
    1.0,
    0.0,
    0.7,
    0.0,
    0.0
  },
  {
    DRUM_MIDTOM,
    MIDI_B3,
    "S_hiTm5",
    S_hiTom_5,
    "B",
    38592,
    1.1,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_HIGHTOM,
    MIDI_C4,
    "S_hiTm5",
    S_hiTom_5,
    "B",
    38592,
    1.3,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_CRASH,
    MIDI_CIS4,
    "S_splsh1",
    S_splash1_1,
    "B",
    57856,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_HIGHTOM,
    MIDI_D4,
    "S_hiTom5",
    S_hiTom_5,
    "B",
    38592,
    1.4,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_CRASH,
    MIDI_DIS4,
    "S_china4",
    S_china1_4,
    "B",
    96832,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_CRASH,
    MIDI_E4,
    "S_china2",
    S_china2_1,
    "B",
    68864,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_RIDE,
    MIDI_F4,
    "S_rBell1",
    S_ride1Bell_1,
    "B",
    86336,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_RIDE,
    MIDI_FIS4,
    "S_rBell1",
    S_ride1Bell_1,
    "B",
    86336,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_CRASH,
    MIDI_G4,
    "S_crash1",
    S_crash1_1,
    "B",
    122112,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_GIS4,
    "S_cowb2",
    S_cowbell_2,
    "B",
    18368,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_CRASH,
    MIDI_A4,
    "S_Crash2",
    S_ride2Crash_2,
    "B",
    143808,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_RIDE,
    MIDI_AIS4,
    "S_ride2",
    S_ride2_1,
    "B",
    132928,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_BASS,
    MIDI_B4,
    "808Kick",
    Kick808,
    "B",
    26688,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_BASS,
    MIDI_C5,
    "phkick1",
    phkick1,
    "B",
    4800,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_CIS5,
    "rims1",
    rims1,
    "R",
    768,
    0.0,
    1.0,
    -0.2,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_D5,
    "Shaker",
    Shaker,
    "R",
    1792,
    0.0,
    1.0,
    -0.2,
    0.7,
    0.0,
    0.0
  },
  {
    DRUM_HANDCLAP,
    MIDI_DIS5,
    "LNclap1",
    LNclap1,
    "C",
    6976,
    0.0,
    1.0,
    0.1,
    1.0,
    0.0,
    0.1
  },
  { //30
    DRUM_SNARE,
    MIDI_E5,
    "LNsnare1",
    LNsnare1,
    "S",
    10560,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_BASS,
    MIDI_F5,
    "Bdtranc",
    Bdtrancy,
    "B",
    7872,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_HIHAT,
    MIDI_FIS5,
    "hhcl1",
    hhcl1,
    "H",
    6208,
    0.0,
    1.0,
    1.0,
    0.6,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_G5,
    "Belltre",
    Belltree,
    "B",
    121408,
    0.0,
    1.0,
    0.0,
    0.7,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_GIS5,
    "Bongo16",
    Bongo16,
    "B",
    6464,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_A5,
    "Bongo27",
    Bongo27,
    "B",
    11712,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_AIS5,
    "Casta",
    Casta,
    "C",
    2048,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_BASS,
    MIDI_B5,
    "Cr78kick",
    Cr78kick,
    "B",
    10048,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_HIHAT,
    MIDI_C6,
    "Cr78tmb1",
    Cr78tmb1,
    "T",
    7552,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_HIHAT,
    MIDI_CIS6,
    "Cr78tmb2",
    Cr78tmb2,
    "T",
    10880,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_CRASH,
    MIDI_D6,
    "Crash1",
    Crash1,
    "B",
    72768,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_DIS6,
    "Dmpop",
    Dmpop,
    "B",
    7808,
    1.0,
    0.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_E6,
    "Electr1",
    Electr1,
    "B",
    2880,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_F6,
    "Excow",
    Excow,
    "B",
    20160,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_FIS6,
    "Tamb",
    Tamb,
    "T",
    8128,
    0.0,
    1.0,
    -0.2,
    0.8,
    0.0,
    0.0
  },
  {
    DRUM_HIHAT,
    MIDI_G6,
    "660HatC1",
    HatC660,
    "T",
    5952,
    0.0,
    1.0,
    0.0,
    0.8,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_GIS6,
    "Cowbell",
    Cowbell,
    "S",
    7296,
    0.0,
    1.0,
    0.2,
    0.8,
    0.0,
    0.0
  },
  {
    DRUM_HIHAT,
    MIDI_A6,
    "Hhopen1",
    Hhopen1,
    "S",
    11200,
    0.0,
    1.0,
    0.0,
    0.8,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_AIS6,
    "Histick",
    Histicks,
    "S",
    2432,
    0.0,
    1.0,
    0.0,
    0.8,
    0.0,
    0.0
  },
  {
    DRUM_SNARE,
    MIDI_B6,
    "Hr16snr2",
    Hr16snr2,
    "S",
    15168,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  { //50
    DRUM_HIGHTOM,
    MIDI_C7,
    "Xrhitom",
    Xrhitom,
    "B",
    13504,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_CIS7,
    "Tick1",
    Tick1,
    "S",
    640,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_D7,
    "M1-18",
    M1_18,
    "S",
    2240,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_HANDCLAP,
    MIDI_DIS7,
    "Md16clp",
    Md16_clp,
    "S",
    13312,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_HIHAT,
    MIDI_E7,
    "Ohhwav",
    Ohhwav,
    "S",
    23488,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_F7,
    "Quijada",
    Quijada,
    "S",
    36096,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_FIS7,
    "tabla1",
    tabla1,
    "S",
    15488,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_RIDE,
    MIDI_G7,
    "Ride808",
    Ride808,
    "S",
    21696,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_PERCUSSION,
    MIDI_GIS7,
    "Scratch",
    Scratch1,
    "S",
    5888,
    0.0,
    1.0,
    0.0,
    0.8,
    0.0,
    0.0
  },
  {
    DRUM_LOWTOM,
    MIDI_A7,
    "Tomlow",
    Tomlow,
    "S",
    15360,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  { //60
    DRUM_LOWTOM,
    MIDI_AIS7,
    "Tom808",
    Tom808,
    "S",
    6272,
    0.0,
    1.0,
    0.0,
    0.9,
    0.0,
    0.0
  },
  {
    DRUM_LOWTOM,
    MIDI_B7,
    "Vl1lbeep",
    Vl1lbeep,
    "B",
    3008,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  {
    DRUM_HIGHTOM,
    MIDI_C8,
    "Vl1hbeep",
    Vl1hbeep,
    "B",
    1984,
    0.0,
    1.0,
    0.0,
    1.0,
    0.0,
    0.0
  },
  { //63
    DRUM_NONE,
    0,
    "EMPTY",
    NULL,
    "-",
    0,
    0.0,
    0.0,
    0.0,
    0.0,
    0.0,
    0.0
  }
};

#endif
